/*
 * Copyright (c) 2003-2007 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _SLDRIVER_CF_H
#define _SLDRIVER_CF_H

#include "drivers.h"

/* Crystalfontz command set */
#define CFCMD_CURSOR_HOME                            1
#define CFCMD_HIDE_DISPLAY                           2
#define CFCMD_RESTORE_DISPLAY                        3
#define CFCMD_HIDE_CURSOR                            4
#define CFCMD_SHOW_UNDERLINE_CURSOR                  5
#define CFCMD_SHOW_BLOCK_CURSOR                      6
#define CFCMD_SHOW_INVERTED_BLOCK_CURSOR             7
#define CFCMD_BACKSPACE                              8
#define CFCMD_CONTROL_BOOT_SCREEN                    9
#define CFCMD_LINE_FEED                              10
#define CFCMD_DELETE_IN_PLACE                        11
#define CFCMD_FORM_FEED                              12
#define CFCMD_CARRIAGE_RETURN                        13
#define CFCMD_BACKLIGHT_CONTROL                      14
#define CFCMD_CONTRAST_CONTROL                       15
#define CFCMD_UNUSED1                                16
#define CFCMD_SET_CURSOR_POSITION                    17
#define CFCMD_HORIZONTAL_BAR_GRAPH                   18
#define CFCMD_SCROLL_ON                              19
#define CFCMD_SCROLL_OFF                             20
#define CFCMD_SET_SCROLLING_MARQUEE_CHARACTER        21
#define CFCMD_ENABLE_MARQUEE                         22
#define CFCMD_WRAP_ON                                23
#define CFCMD_WRAP_OFF                               24
#define CFCMD_SET_CUSTOM_CHARACTER_BITMAP            25
#define CFCMD_REBOOT                                 26
#define CFCMD_ESCAPE                                 27
#define CFCMD_LARGE_BLOCK_NUMBER                     28
#define CFCMD_UNUSED2                                29
#define CFCMD_SEND_RAW                               30
#define CFCMD_SHOW_INFORMATION_SCREEN                31

/* default settings */
#define CF_DEFAULT_CONTRAST                          70
#define CF_DEFAULT_BACKLIGHT                         100

/* special characters */
#define CF_CHAR_SPACE                                0x20
#define CF_CHAR_SPECIAL                              0x80
#define CF_CHAR_BLOCK                                0x80
#define CF_CHAR_VBAR                                 0x81
#define CF_CHAR_HBAR                                 0x81
#define CF_SPECIAL                                   CF_CHAR_SPECIAL
#define CF_SPECIAL_END                               CF_CHAR_SPECIAL + 7
#define CF_SPECIAL_WIDTH                             6
#define CF_SPECIAL_HEIGTH                            8

/* functions */
__BEGIN_DECLS
void cf_connect(struct driver *);
void cf_disconnect(struct driver *);
int cf_isconnected(struct driver *);
void cf_clear(struct driver *);
char cf_gethchar(struct driver *, int);
char *cf_getoption();
char cf_getvchar(struct driver *, int);
void cf_flush(struct driver *);
void cf_nprintf(struct driver *, int, char *, ...);
void cf_option(struct driver *, int, char **);
void cf_printf(struct driver *, char *fmt, ...);
void cf_reset(struct driver *);
void cf_setbacklight(struct driver *, int);
void cf_setcapabilities(struct driver *, int);
void cf_setcontrast(struct driver *, int);
void cf_setspecialchar(struct driver *, int, struct specialchar *);
void cf_setxy(struct driver *, int, int);
void cf_setupblock(struct driver *);
void cf_setuphbar(struct driver *);
void cf_setupvbar(struct driver *);
__END_DECLS
#endif
