/*
 * Copyright (c) 2007 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _SLDRIVER_CFP_H
#define _SLDRIVER_CFP_H

#include "drivers.h"

/* Crystalfontz packet command set */
#define CFPCMD_PING                                  0
#define CFPCMD_GET_VERSION                           1
#define CFPCMD_WRITE_USER_FLASH_AREA                 2
#define CFPCMD_READ_USER_FLASH_AREA                  3
#define CFPCMD_SAVE_BOOT_STATE                       4
#define CFPCMD_RESET                                 5
#define CFPCMD_CLEAR                                 6
#define CFPCMD_UNUSED1                               7
#define CFPCMD_UNUSED2                               8
#define CFPCMD_SET_SPECIAL_CHARACTER_DATA            9
#define CFPCMD_READ_MEMORY                           10
#define CFPCMD_SET_CURSOR_POSITION                   11
#define CFPCMD_SET_CURSOR_STYLE                      12
#define CFPCMD_SET_CONTRAST                          13
#define CFPCMD_SET_BACKLIGHT                         14
#define CFPCMD_UNUSED3                               15
#define CFPCMD_SET_FAN_REPORTING                     16
#define CFPCMD_SET_FAN_POWER                         17
#define CFPCMD_READ_DOW_DEVICE_INFORMATION           18
#define CFPCMD_SET_TEMPERATURE_REPORTING             19
#define CFPCMD_ARBITRARY_DOW_TRANSACTION             20
#define CFPCMD_UNUSED4                               21
#define CFPCMD_SEND_LCD_CONTROLLER                   22
#define CFPCMD_SET_KEY_REPORTING                     23
#define CFPCMD_READ_KEYPAD                           24
#define CFPCMD_SET_FAN_POWER_FAILSAFE                25
#define CFPCMD_SET_FAN_TACHOMETER_GLITCH_FILTER      26
#define CFPCMD_GET_FAN_POWER_AND_FAILSAFE            27
#define CFPCMD_SET_ATX_POWER_SWITCH                  28
#define CFPCMD_SET_WATCHDOG                          29
#define CFPCMD_GET_REPORTING                         30
#define CFPCMD_PRINT                                 31
#define CFPCMD_UNUSED5                               32 /* TODO: determine what these do */
#define CFPCMD_SET_BAUD_RATE                         33
#define CFPCMD_SET_GPIO                              34
#define CFPCMD_GET_GPIO                              35

#define CFPARG_RESET_LCD_0                           8
#define CFPARG_RESET_LCD_1                           18
#define CFPARG_RESET_LCD_2                           99

#define CFPARG_RESET_HOST_0                          12
#define CFPARG_RESET_HOST_1                          28
#define CFPARG_RESET_HOST_2                          97

#define CFPARG_POWER_OFF_HOST_0                      3
#define CFPARG_POWER_OFF_HOST_1                      11
#define CFPARG_POWER_OFF_HOST_2                      95

/* default settings */
#define CFP_DEFAULT_CONTRAST                         100
#define CFP_DEFAULT_BACKLIGHT                        100

/* special characters */
#define CFP_CHAR_SPACE                               0x20
#define CFP_CHAR_SPECIAL                             0x00
#define CFP_CHAR_BLOCK                               0x07
#define CFP_CHAR_VBAR                                0x00
#define CFP_CHAR_HBAR                                0x00
#define CFP_SPECIAL                                  CFP_CHAR_SPECIAL
#define CFP_SPECIAL_END                              CFP_CHAR_SPECIAL + 7
#define CFP_SPECIAL_WIDTH                            6
#define CFP_SPECIAL_HEIGTH                           8

/* functions */
__BEGIN_DECLS
void cfp_connect(struct driver *);
void cfp_disconnect(struct driver *);
int cfp_isconnected(struct driver *);
void cfp_clear(struct driver *);
char cfp_gethchar(struct driver *, int);
char *cfp_getoption();
char cfp_getvchar(struct driver *, int);
void cfp_flush(struct driver *);
void cfp_nprintf(struct driver *, int, char *, ...);
void cfp_option(struct driver *, int, char **);
void cfp_printf(struct driver *, char *fmt, ...);
void cfp_reset(struct driver *);
void cfp_setbacklight(struct driver *, int);
void cfp_setcapabilities(struct driver *, int);
void cfp_setcontrast(struct driver *, int);
void cfp_setspecialchar(struct driver *, int, struct specialchar *);
void cfp_setxy(struct driver *, int, int);
void cfp_setupblock(struct driver *);
void cfp_setuphbar(struct driver *);
void cfp_setupvbar(struct driver *);
__END_DECLS
#endif
