/*
 * Copyright (c) 2005-2007 Willem Dijkstra
 *                         Jorge Sanchez Pelaez beto
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _SLDRIVER_HD44780_H
#define _SLDRIVER_HD44780_H

#include "drivers.h"
#include "port.h"

/* HD44780 command set */
#define HDCMD_CLEAR_DISPLAY                        0x01
#define HDCMD_CURSOR_HOME                          0x02
#define HDCMD_ENTRY_MODE                           0x04
#define HDCMD_ENTRY_MODE_CURSOR_DECREMENT          0x00
#define HDCMD_ENTRY_MODE_CURSOR_INCREMENT          0x02
#define HDCMD_ENTRY_MODE_DISPLAY_SHIFT             0x01
#define HDCMD_DISPLAY_CONTROL                      0x08
#define HDCMD_DISPLAY_CONTROL_DISPLAY_OFF          0x00
#define HDCMD_DISPLAY_CONTROL_DISPLAY_ON           0x04
#define HDCMD_DISPLAY_CONTROL_CURSOR_OFF           0x00
#define HDCMD_DISPLAY_CONTROL_CURSOR_ON            0x02
#define HDCMD_DISPLAY_CONTROL_BLINK_OFF            0x00
#define HDCMD_DISPLAY_CONTROL_BLINK_ON             0x01
#define HDCMD_SHIFT                                0x10
#define HDCMD_SHIFT_DISPLAY                        0x08
#define HDCMD_SHIFT_CURSOR                         0x00
#define HDCMD_SHIFT_RIGHT                          0x04
#define HDCMD_SHIFT_LEFT                           0x00
#define HDCMD_FUNCTION                             0x20
#define HDCMD_FUNCTION_4BIT                        0x00
#define HDCMD_FUNCTION_8BIT                        0x10
#define HDCMD_FUNCTION_1LINE                       0x00
#define HDCMD_FUNCTION_2LINE                       0x08
#define HDCMD_FUNCTION_FONT5X7                     0x00
#define HDCMD_FUNCTION_FONT5X10                    0x04
#define HDCMD_SETCGRAM                             0x40
#define HDCMD_SETDDRAM                             0x80

/* control pin mappings */
#define HDPIN_ENABLE                               LPT_C0
#define HDPIN_RW                                   LPT_C1
#define HDPIN_RS                                   LPT_C2

/* special characters */
#define HD_CHAR_SPACE                              0x20
#define HD_CHAR_SPECIAL                            0x01
#define HD_CHAR_BLOCK                              0x01
#define HD_CHAR_VBAR                               0x02
#define HD_CHAR_HBAR                               0x02
#define HD_SPECIAL                                 HD_CHAR_SPECIAL
#define HD_SPECIAL_END                             HD_CHAR_SPECIAL + 8
#define HD_SPECIAL_WIDTH                           5
#define HD_SPECIAL_HEIGTH                          7

#define HD_ROWSTART                                {0x00, 0x40, 0x14, 0x54}

/* functions */
__BEGIN_DECLS
void hd44780_connect(struct driver *);
void hd44780_disconnect(struct driver *);
int hd44780_isconnected(struct driver *);
void hd44780_clear(struct driver *);
char hd44780_gethchar(struct driver *, int);
char *hd44780_getoption();
char hd44780_getvchar(struct driver *, int);
void hd44780_flush(struct driver *);
void hd44780_nprintf(struct driver *, int, char *, ...);
void hd44780_option(struct driver *, int, char **);
void hd44780_printf(struct driver *, char *fmt, ...);
void hd44780_reset(struct driver *);
void hd44780_setbacklight(struct driver *, int);
void hd44780_setcapabilities(struct driver *, int);
void hd44780_setcontrast(struct driver *, int);
void hd44780_setspecialchar(struct driver *, int, struct specialchar *);
void hd44780_setxy(struct driver *, int, int);
void hd44780_setupblock(struct driver *);
void hd44780_setuphbar(struct driver *);
void hd44780_setupvbar(struct driver *);
__END_DECLS

#endif
