/*
 * Copyright (c) 2004-2005 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "drivers.h"
#include "error.h"
#include "driver_test.h"

void
test_connect(struct driver *d)
{
    info("test_connect");
}
void
test_disconnect(struct driver *d)
{
    info("test_disconnect");
}
int
test_isconnected(struct driver *d)
{
    return 1;
}
void
test_clear(struct driver *d)
{
    info("test_clear");
}
char
test_gethchar(struct driver *d, int percentage)
{
    char *hchars = "abcdefghij";

    percentage %= 101;
    percentage /= 10;

    return hchars[percentage];
}
char *
test_getoption(struct driver *d)
{
    return NULL;
}
char
test_getvchar(struct driver *d, int percentage)
{
    char *vchars = "0123456789";

    percentage %= 101;
    percentage /= 10;

    return vchars[percentage];
}
void
test_flush(struct driver *d)
{
    info("test_flush");
}
void
test_nprintf(struct driver *d, int maxsize, char *fmt, ...)
{
    va_list args;
    int size;

    maxsize %= d->buflen;

    va_start(args, fmt);
    size = vsnprintf(d->buffer, maxsize, fmt, args);
    va_end(args);

    info("%02d,%02d:%s", d->x, d->y, d->buffer);
}
void
test_option(struct driver *d, int argc, char **argv)
{
    return;
}
void
test_printf(struct driver *d, char *fmt, ...)
{
    va_list args;
    int size;

    va_start(args, fmt);
    size = vsnprintf(d->buffer, d->buflen, fmt, args);
    va_end(args);

    info("%02d,%02d:%s", d->x, d->y, d->buffer);
}
void
test_reset(struct driver *d)
{
    info("test_reset");
}
void
test_setbacklight(struct driver *d, int backlight)
{
    info("test_setbacklight %d", backlight);
}
void
test_setcapabilities(struct driver *d, int capabilities)
{
    info("test_setcapabilities 0x%08x", capabilities);
}
void
test_setcontrast(struct driver *d, int contrast)
{
    info("test_setcontrast %d", contrast);
}
void
test_setspecialchar(struct driver *d, int ch, struct specialchar *data)
{
    info("test_setspecialchar %d", ch);
}
void
test_setupblock(struct driver *d)
{
    info("test_setupblock");
}
void
test_setuphbar(struct driver *d)
{
    info("test_setuphbar");
}
void
test_setupvbar(struct driver *d)
{
    info("test_setupvbar");
}
void
test_setxy(struct driver *d, int x, int y)
{
    d->x = x % d->maxx;
    d->y = y % d->maxy;
}
