/*
 * Copyright (c) 2005-2007 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _SLDRIVERS_H
#define _SLDRIVERS_H

#include "cfp_packet.h"
#include "rbuffer.h"

/* driver capabilities */
#define DC_BACKLIGHT    0x00000001
#define DC_BLOCK        0x00000002
#define DC_CONTRAST     0x00000004
#define DC_HBAR         0x00000008
#define DC_VBAR         0x00000010

struct driverfmap;

struct driver {
    char *connect;
    char *driver;
    char *model;
    char *buffer;
    unsigned int buflen;
    unsigned int x, y;
    unsigned int maxx, maxy;
    unsigned int capabilities;
    unsigned int speed;
    struct driverfmap *fmap;
    union {
	struct {
	    int fd;
	} cf;

	struct {
	    int fd;
            struct cfpacket p;
	    struct rbuffer rb;
	} cfp;

	struct {
	    char rowaddr[4];
	    int dataport;
	    int controlport;
	} hd44780;
    } control;
};

struct specialchar {
    unsigned long line[8];
};

struct model {
    char *name;
    char *driver;
    unsigned int maxx, maxy;
    unsigned int speed;
    struct {
        unsigned int start;
        unsigned int stop;
        unsigned int height;
        unsigned int width;
    } special;
};

struct driverfmap {
    char *name;

    /* connection primitives */
    void (*connect) (struct driver *);
    void (*disconnect) (struct driver *);
    int (*isconnected) (struct driver *);

    /* lower order driver primitives */
    void (*clear) (struct driver *);
    char (*gethchar) (struct driver *, int percentage);
    char (*getvchar) (struct driver *, int percentage);
    void (*flush) (struct driver *);
    void (*nprintf) (struct driver *, int maxsize, char *fmt, ...);
    void (*printf) (struct driver *, char *fmt, ...);
    void (*reset) (struct driver *);
    void (*setbacklight) (struct driver *, int percentage);
    void (*setcapabilities) (struct driver *, int capabilities);
    void (*setcontrast) (struct driver *, int percentage);
    void (*setspecialchar) (struct driver *, int ch, struct specialchar *data);
    void (*setupblock) (struct driver *);
    void (*setuphbar) (struct driver *);
    void (*setupvbar) (struct driver *);
    void (*setxy) (struct driver *, int x, int y);

    /* driver specific extras */
    char *(*getoption) ();
    void (*option) (struct driver *, int, char **);
};


struct driver *getdriver(char *model, char *connectstring);
int getmodelinfo(char *modelname, char *buffer, size_t maxsize);
int getmodelinfoheader(char *buffer, size_t maxsize);
int getmodels(char *buffer, size_t maxsize);

#endif
