/*
 * Copyright (c) 2005 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "error.h"
#include "conf.h"

#ifdef HAS_INB_SYS_IO_H
#include <sys/io.h>
#else
#ifdef HAS_INB_MACH_PIO_H
#include <sys/types.h>
#include <machine/pio.h>
#include <machine/sysarch.h>
#else
#ifdef HAS_INB_MACH_CPUFUNC_H
#include <sys/types.h>
#include <machine/cpufunc.h>
#include <machine/sysarch.h>
#endif /* HAS_INB_MACH_CPUFUNC_H */
#endif /* HAS_INB_MACH_PIO_H */
#endif /* HAS_INB_SYS_IO_H */

int
readport(unsigned short int port)
{
#ifdef HAS_INB
    return inb(port);
#else
    fatal("inb support not found at compile time - cannot use parallel ports");
#endif /* HAS_INB */
}

void
writeport(unsigned short int port, unsigned char val) {
#ifdef HAS_INB_SYS_IO_H
    outb(val, port);
#else
#ifdef HAS_INB
    outb(port, val);
#else
    fatal("inb support not found at compile time - cannot use parallel ports");
#endif /* HAS_INB */
#endif /* HAS_INB_SYS_IO_H */
}

void
accessport (unsigned short int port)
{
#ifdef HAS_INB_SYS_IO_H
    if (ioperm(port, 1, 255) == -1) {
	fatal("cannot get io permissions for port 0x%x: %.200s",
	      port, strerror(errno));
    }
#else
#ifdef HAS_INB_MACH_PIO_H
    unsigned long iomap[32];

    if (i386_get_ioperm(iomap) == -1) {
	fatal("cannot get io permissions for port 0x%x: %.200s",
	      port, strerror(errno));
    }

    iomap[port >> 5] &= ~(1 << (port & 0x1f));

    if (i386_set_ioperm(iomap) == -1) {
	fatal("cannot set io permissions for port 0x%x: %.200s",
	      port, strerror(errno));
    }
#else
#ifdef HAS_INB_MACH_CPUFUNC_H
    if (i386_set_ioperm(port, 1, 1) == -1) {
	fatal("cannot set io permissions for port 0x%x: %.200s",
	      port, strerror(errno));
    }
#else
    fatal("inb support not found at compile time - cannot use parallel ports");
#endif /* HAS_INB_MACH_CPUFUNC_H */
#endif /* HAS_INB_MACH_PIO_H */
#endif /* HAS_INB_SYS_IO_H */
}
