/*
 * Copyright (c) 2007 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _SLNRBUFFER_H
#define _SLNRBUFFER_H

#include <sys/time.h>

#include <stdint.h>

#define RBUFFER_SIZE    20480
struct rbuffer {
    uint32_t fd;
    uint32_t start;
    uint32_t end;
    char data[RBUFFER_SIZE];
};

__BEGIN_DECLS
void rbuffer_attach(struct rbuffer *rb, uint32_t fd);
void rbuffer_clear(struct rbuffer *rb);
void rbuffer_compact(struct rbuffer *rb);
uint32_t rbuffer_fd(struct rbuffer *rb);
uint32_t rbuffer_get_byte(struct rbuffer *rb, uint8_t *byte);
uint32_t rbuffer_get_data(struct rbuffer *rb, size_t *len);
uint32_t rbuffer_get_data_wait(struct rbuffer *rb, size_t *len, struct timeval *wait);
uint32_t rbuffer_get_line(struct rbuffer *rb, char **s, size_t *maxlen);
uint32_t rbuffer_len(struct rbuffer *rb);
uint32_t rbuffer_unget_byte(struct rbuffer *rb, uint8_t byte);
__END_DECLS

#endif
