/*
 * Copyright (c) 2003 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <limits.h>
#include <stdio.h>
#include <string.h>

#include "stream.h"
#include "xmalloc.h"

int
is_host_stream(char *s, size_t maxlen, char *host)
{
    if (strncmp(s, host, strlen(host)) != 0)
	return 0;
    else
	return 1;
}

/* Return the index of stream <stype>(<sargs>) for <host> */
int
get_stream_index(char *s, size_t maxlen, char *stype, char *sargs)
{
    char search[_POSIX2_LINE_MAX];
    char *p;
    int len;

    if (sargs != NULL) {
	snprintf(search, sizeof(search), ";%s:%s:", stype, sargs);
    } else {
	snprintf(search, sizeof(search), ";%s:", stype);
    }

    p = strstr(s, search);

    if (p == NULL)
	return -1;
    else {
	p++;
	len = (p - s);
	if (len < maxlen)
	    return len;
	else
	    return 0;
    }
}

int
get_nextarg_index(char *s, size_t maxlen, int item)
{
    int i;

    for (i = 0; i < maxlen && item > 0; i++) {
	if (s[i] == ';') {
	    /* end of stream */
	    return 0;
	}
	if (s[i] == ':') {
	    /* end of item */
	    item--;
	}
    }

    if (i == maxlen)
	return 0;
    else
	return i;
}

struct stream *
create_stream(char *host, char *stype, char *sargs)
{
    struct stream *p;

    if (host == NULL || stype == NULL || sargs == NULL)
	return NULL;

    p = (struct stream *) xmalloc(sizeof(struct stream));
    bzero(p, sizeof(struct stream));

    p->host  = xstrdup(host);
    p->stype = xstrdup(stype);
    p->sargs = xstrdup(sargs);

    return p;
}

struct stream *
find_stream(struct streamlist *sl, char *host, char *stype, char *sargs)
{
    struct stream *p;

    if (sl == NULL)
	return NULL;

    SLIST_FOREACH(p, sl, next) {
	if (((p->host  != NULL && host  != NULL && strncmp(host, p->host, strlen(p->host))    == 0) ||
	     (p->host  == NULL && host  == NULL)) &&
	    ((p->stype != NULL && stype != NULL && strncmp(stype, p->stype, strlen(p->stype)) == 0) ||
	     (p->stype == NULL && stype == NULL)) &&
	    ((p->sargs != NULL && sargs != NULL && strncmp(sargs, p->sargs, strlen(p->sargs)) == 0) ||
	     (p->sargs == NULL && sargs == NULL)))
	    return p;
    }

    return NULL;
}
